/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/


#include "CollectionView/collectionscene.h"

#include "button_item.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionModel/histomodel.h"
#include "Playlist/playlistmodel.h"

#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"
#include "MediaItem/covercache.h"

#include "Database/database.h"
#include "Database/databasemanager.h"
#include "Histo/histomanager.h"

#include "Dialog/mediaitem_edit_dialog.h"
#include "SmartPlaylist/smartplaylist.h"
#include "Playlist/playlisteditor.h"

#include "threadmanager.h"
#include "yarocksettings.h"
#include "utilities.h"
#include "global_actions.h"
#include "debug.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>


/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*                                                                              *
********************************************************************************
*/
CollectionScene::CollectionScene(QWidget *parent) : QGraphicsScene(parent)
{
    _parentWidget = parent;
    this->setParent(_parentWidget);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setStickyFocus(true);

    //! set model
    _collectionModel = CollectionModel::instance();
    _plsFileModel    = PlsFileModel::instance();
    _histoModel      = HistoModel::instance();

    connect(_collectionModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_plsFileModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_histoModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));

    m_infosize           = 0;
    _mouseGrabbedItem    = 0;

    //! graphic item context menu
    m_graphic_item_menu = new GraphicsItemMenu(0);
    m_graphic_item_menu->setScene(this);

    QObject::connect(m_graphic_item_menu, SIGNAL(menu_action_triggered(ENUM_ACTION_ITEM_MENU)), this, SLOT(slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU)), Qt::DirectConnection);

    /*  scene actions */
    m_actions.insert("album_grid", new QAction(QIcon(),QString(tr("view grid")),this));
    m_actions.insert("playlist_tracks", new QAction(QIcon(),QString(tr("view tracks")),this));
    m_actions.insert("new_playlist",new QAction(QIcon(":/images/document-new-48x48.png"),QString(tr("new playlist")),this));
    m_actions.insert("new_smart",new QAction(QIcon(":/images/document-new-48x48.png"),QString(tr("new smart playlist")),this));
    m_actions.insert("cover", ACTIONS()->value(TASK_COVER_SEARCH));
    m_actions.insert("reload_histo", new QAction(QIcon(":/images/rebuild.png"),QString(tr("reload history")),this));
    m_actions.insert("clear_histo", new QAction(QIcon(),QString(tr("clear history")),this));
    
    m_actions.value("album_grid")->setCheckable(true);
    m_actions.value("album_grid")->setChecked( SETTINGS()->_album_view_type == 0);
    
    m_actions.value("playlist_tracks")->setCheckable(true);
    m_actions.value("playlist_tracks")->setChecked(SETTINGS()->_playlist_view_type ==1);

    connect(m_actions.value("album_grid"), SIGNAL(triggered()), this, SLOT(slot_change_view_settings()));
    connect(m_actions.value("playlist_tracks"), SIGNAL(triggered()), this, SLOT(slot_change_view_settings()));
    connect(m_actions.value("new_playlist"), SIGNAL(triggered()), this, SLOT(slot_create_new_playlist()));
    connect(m_actions.value("new_smart"), SIGNAL(triggered()), this, SLOT(slot_create_new_smart_playlist()));
    connect(m_actions.value("reload_histo"), SIGNAL(triggered()), this, SLOT(populateScene()));
    connect(m_actions.value("clear_histo"), SIGNAL(triggered()), this, SLOT(slot_clear_history()));
        
    //! gestion du double click par action globale et data
    ACTIONS()->insert(BROWSER_ITEM_RATING_CLICK, new QAction(this));
    ACTIONS()->insert(BROWSER_ITEM_MOUSE_MOVE, new QAction(this));
    connect(ACTIONS()->value(BROWSER_ITEM_MOUSE_MOVE), SIGNAL(triggered()), this, SLOT(slot_item_mouseMove()), Qt::DirectConnection);
    connect(ACTIONS()->value(BROWSER_ITEM_RATING_CLICK), SIGNAL(triggered()), this, SLOT(slot_item_ratingclick()), Qt::DirectConnection);
}

CollectionScene::~CollectionScene()
{
    clear();
}


//! --------------------- Change view ------------------------------------------
void CollectionScene::slot_change_view_settings()
{
    SETTINGS()->_album_view_type    = m_actions.value("album_grid")->isChecked() ? 0 : 1;
    SETTINGS()->_playlist_view_type = m_actions.value("playlist_tracks")->isChecked() ? 1 : 0;
    
    populateScene();
}

/*******************************************************************************
    Collection Scene Populator
*******************************************************************************/
void CollectionScene::populateScene()
{
    Debug::debug() << "CollectionScene::PopulateScene";

    //! clear scene and delete all items
    clear();
    switch(m_view_mode)
    {
      case MAIN::ViewAlbum          :
        if( MAIN::ViewAlbum_Type(SETTINGS()->_album_view_type) == MAIN::album_grid)
          populateAlbumGridScene();
        else
          populateAlbumScene();
      break;

      case MAIN::ViewArtist         : populateArtistScene(); break;
      case MAIN::ViewTrack          : populateTrackScene(); break;
      case MAIN::ViewGenre          : populateGenreScene(); break;
      case MAIN::ViewYear           : populateYearScene(); break;
      case MAIN::ViewFavorite       : populateFavoriteScene(); break;
      case MAIN::ViewPlaylist       :
        if( MAIN::ViewPlaylist_Type(SETTINGS()->_playlist_view_type) == MAIN::playlist_overview)
          populatePlaylistOverviewScene();
        else
          populatePlaylistByTrackScene();
        break;
      case MAIN::ViewSmartPlaylist  : populatePlaylistSmartScene(); break;
      case MAIN::ViewHistory        : populateHistoScene(); break;
      case MAIN::ViewDashBoard      : populateDashBoardScene(); break;
      default: break;
    }

    //! we need to ajust SceneRect
    int diff = _parentWidget->height() - itemsBoundingRect().height();
    int ajust  = ( diff > 0) ? diff : 0;
    setSceneRect ( itemsBoundingRect().adjusted(-10, -10, 0, ajust-10) );
}


void CollectionScene::populateAlbumScene()
{
    int albumRow  = 0;
    int artistRow = 0;
    int Column    = 0;
    m_infosize    = 0;

    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();
    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth-10;
    
    header->setText( tr("Albums") );
    header->setPos(0, 5);
    addItem(header);    
    
    //!  menu item
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("album_grid"));
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    artistRow++;

    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ )
    {
      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast( _collectionModel->rootItem()->child(i) );

      if(!_collectionModel->isArtistFiltered(artist) ) continue;

      CategorieGraphicItem *category = new CategorieGraphicItem();
      category->_catName = artist->name;
      category->_width   = _parentWidget->width() -20;
      category->setPos( 0 ,10 + artistRow*50 + albumRow*170);

      addItem(category);

      Column = 0;
      artistRow++;

      //! album loop
      for (int j = 0; j < artist->childCount(); j++)
      {
        MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast( artist->child(j) );
        if(!_collectionModel->isAlbumFiltered(album) ) continue;

        m_infosize++;

        AlbumGraphicItem_v2 *album_item = new AlbumGraphicItem_v2();
        album_item->media = album;
        album_item->setPos(4+160*Column, artistRow*50 + albumRow*170);
        addItem(album_item);


        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          albumRow++;
        }
      }
      if(Column>0) albumRow++;
    }

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + artistRow*50 + albumRow*20);
      addItem(info);
    }
}

void CollectionScene::populateAlbumGridScene()
{
    int albumRow      = 0;
    int Column        = 0;
    m_infosize        = 0;
    int categorieRow  = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Albums") );
    header->setPos(0, 5);
    addItem(header);    
    
    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("album_grid"));
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    categorieRow++;

    CategorieGraphicItem *cat = new CategorieGraphicItem();
    cat->_catName = tr("All Albums");
    cat->_width   = _parentWidget->width() - 20;

    cat->setPos( 0 ,10 + categorieRow*50);
    categorieRow++;
    addItem(cat);


    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ )
    {
      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));

      if(!_collectionModel->isArtistFiltered(artist) ) continue;

      //! album loop
      for (int j = 0; j < artist->childCount(); j++)
      {
        MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast( artist->child(j) );
        if(!_collectionModel->isAlbumFiltered(album) ) continue;

        m_infosize++;

        AlbumGraphicItem *album_item = new AlbumGraphicItem();
        album_item->media = album;
        album_item->setPos(4+160*Column, 10 + albumRow*170 + categorieRow*50);
        addItem(album_item);

        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          albumRow++;
        }
      }
    }

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 ,10 + categorieRow*50);
      addItem(info);
    }
}



void CollectionScene::populateArtistScene()
{
    int artistRow    = 0;
    int categorieRow = 0;
    int Column       = 0;
    m_infosize       = 0;
    int idx          = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();
    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Artists") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);
    
    categorieRow++;
        
    //! artist loop
    QChar start_char;
    for (int i=0 ; i < _collectionModel->rootItem()->childCount(); i++ )
    {
        MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));
        if(! _collectionModel->isArtistFiltered(artist) ) continue;

        QChar current_char = artist->name.at(0).toLower();

        if(start_char !=  current_char) {
          //! new category
          start_char = current_char;
          if(idx>0) artistRow++;

          CategorieGraphicItem *category = new CategorieGraphicItem();
          category->_catName = QString(start_char);
          category->_width   = _parentWidget->width() - 20;
          category->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
          addItem(category);

          Column = 0;
          categorieRow++;
        }

        //! add new artist item
        m_infosize++;
        idx++;

        //! New Artist Item
        ArtistGraphicItem *artist_item = new ArtistGraphicItem();
        artist_item->media = artist;
        artist_item->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
        addItem(artist_item);

        //! ALBUM COVER LOOP
        artist->album_covers.clear();
        for(int j = artist->childCount()-1 ; j >= 0; j--) {
          MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(artist->child(j));
          if(!_collectionModel->isAlbumFiltered(album) ) continue;

          artist->album_covers.prepend(album);

          //!WARNING limite de l'affichage à 6 cover max
          if(artist->album_covers.size() >=6) break;
        }

        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          artistRow++;
          idx    = 0;
        }

    } // end for artist loop

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50);
      addItem(info);
    }
}



void CollectionScene::populateTrackScene()
{
    int artistRow     = 0;
    int albumRow      = 0;
    int trackRow      = 0;
    int trackPerAlbum = 0;
    int offset        = 0;
    m_infosize        = 0;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Tracks") );
    header->setPos(0, 5);
    addItem(header);
    
    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    artistRow++;

    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ )
    {
      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));

      if(!_collectionModel->isArtistFiltered(artist) ) continue;

      CategorieGraphicItem *category = new CategorieGraphicItem();
      category->_catName = artist->name;
      category->_width   = _parentWidget->width() - 20;
      category->setPos( 0 ,artistRow*50 + trackRow*20 + albumRow*30 + offset );
      addItem(category);

      artistRow++;

      //! album loop
      for (int j = 0; j < artist->childCount(); j++)
      {
        MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(artist->child(j));

        if(!_collectionModel->isAlbumFiltered(album)) continue;

        AlbumGraphicItem_v2 *album_item = new AlbumGraphicItem_v2();
        album_item->setFlag(QGraphicsItem::ItemIsSelectable, false);
        album_item->media = album;
        album_item->setPos(4, artistRow*50 + trackRow*20 + albumRow*30 + offset );
        addItem(album_item);

        trackPerAlbum = 0;

        //! track loop
        for (int k = 0; k < album->childCount(); k++)
        {
          MEDIA::TrackPtr track = MEDIA::TrackPtr::staticCast(album->child(k));
          if(!_collectionModel->isTrackFiltered(track)) continue;

          TrackGraphicItem_v2 *track_item = new TrackGraphicItem_v2();
          track_item->media = track;
          track_item->setPos(155, artistRow*50 + trackRow*20 + albumRow*30 + offset);

          //PATCH (-20 => fix alignement of the scene)
          track_item->_width = _parentWidget->width()-155-20;

          addItem(track_item);

          trackRow++;
          trackPerAlbum++;
          m_infosize++;
        }
        albumRow++;

        if( trackPerAlbum < 8 ) offset = offset + (8 - trackPerAlbum)*20;
      }
    }

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + artistRow*50);
      addItem(info);
    }
}



void CollectionScene::populateGenreScene()
{
    int categorieRow = 0;
    int albumRow     = 0;
    int Column       = 0;
    m_infosize       = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Albums by genre") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    categorieRow++;

    QString s_album     = "";
    QString s_genre     = "";

    QList<MEDIA::TrackPtr> list_track_by_genre = _collectionModel->trackItemHash.values();


    //! Sort Media Track Item list By Genre
    qSort(list_track_by_genre.begin(), list_track_by_genre.end(),MEDIA::compareTrackItemGenre);

    foreach (MEDIA::TrackPtr track, list_track_by_genre)
    {
      //! add filtre
      if(s_genre == track->genre && s_album == track->album) continue;
      if(!_collectionModel->isTrackFiltered(track)) continue;

      if(s_album != track->album)
      {
        /* ------- New Genre ------- */
        if(s_genre != track->genre)
        {
          s_genre = track->genre;

          if(Column>0) albumRow++;

          CategorieGraphicItem *category = new CategorieGraphicItem();
          category->_catName = s_genre;
          category->_width   = _parentWidget->width()-20;
          category->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
          addItem(category);

          m_infosize++; // on compte les categorie = genre
          categorieRow++;
          Column     = 0;
        }

        /* ------- New Album ------- */
        s_album = track->album;

        AlbumGenreGraphicItem *album_item = new AlbumGenreGraphicItem();
        album_item->media    = MEDIA::AlbumPtr::staticCast(track->parent());
        album_item->_genre   = s_genre;
        album_item->setPos(4+160*Column, categorieRow*50 + albumRow*170);
        addItem(album_item);

        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          albumRow++;
        }
      }
    } // fin foreach track

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50);
      addItem(info);
    }
}


void CollectionScene::populateYearScene()
{
    int categorieRow = 0;
    int albumRow     = 0;
    int Column       = 0;
    m_infosize       = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Albums by year") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    categorieRow++;

    int year         = -1;
    int idx          = 0;

    QList<MEDIA::AlbumPtr> list_album_by_year = _collectionModel->albumItemList;

    //! Sort Media Album Item list By Year
    qSort(list_album_by_year.begin(), list_album_by_year.end(),MEDIA::compareAlbumItemYear);

    foreach (MEDIA::AlbumPtr album, list_album_by_year)
    {
      //! add filtre
      if(!_collectionModel->isAlbumFiltered(album)) continue;

      if(year != album->year) {
        //! new year
        year = album->year;

        if(idx>0) albumRow++;

        CategorieGraphicItem *category = new CategorieGraphicItem();
        category->_catName = QString::number(year);
        category->_width   = _parentWidget->width() -20;
        category->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
        addItem(category);

        m_infosize++; // on compte les categorie = genre
        categorieRow++;
        Column     = 0;
        idx        = 0;
      }

      AlbumGraphicItem *album_item = new AlbumGraphicItem();
      album_item->media  = album;
      album_item->setPos(4+160*Column, categorieRow*50 + albumRow*170);
      addItem(album_item);

      if(Column < (nbItem-1)) {
        Column++;
        idx++;
      }
      else {
        Column = 0;
        albumRow++;
        idx=0;
      }
    } // fin foreach track


    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50);
      addItem(info);
    }
}



void CollectionScene::populateFavoriteScene()
{
    int categorieRow = 0;
    int artistRow    = 0;
    int albumRow     = 0;
    int Column       = 0;
    m_infosize        = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Favorites") );
    header->setPos(0, 5);
    addItem(header);    
    
    //! menu button
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);
    
    categorieRow++;
    
    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = tr("Artists");
    category->_width   = _parentWidget->width()-20;
    category->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(category);

    categorieRow++;


    /*-----------------------------------------------*/
    /* Favorite Artists                              */
    /* ----------------------------------------------*/
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) {

      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));

      if(!artist->isFavorite) continue;
      if(!_collectionModel->isArtistFiltered(artist)) continue;

      m_infosize++; // we count artist favorite item

      //! New Artist Item
      ArtistGraphicItem *artist_item = new ArtistGraphicItem();
      artist_item->media = artist;
      artist_item->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
      addItem(artist_item);

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        artistRow++;
      }

      //! ALBUM COVER LOOP
      artist->album_covers.clear();
      for(int j = artist->childCount()-1 ; j >= 0; j--) {
          MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(artist->child(j));
          if(!_collectionModel->isAlbumFiltered(album) ) continue;

          artist->album_covers.prepend(album);

          //!WARNING limite de l'affichage à 6 cover max
          if(artist->album_covers.size() >=6) break;
      }
    } // end artist loop

    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(info);
      categorieRow++;
    }


    /*-----------------------------------------------*/
    /* Favorite Albums                               */
    /* ----------------------------------------------*/
    if (Column > 0) artistRow++;

    CategorieGraphicItem *category2 = new CategorieGraphicItem();
    category2->_catName = tr("Albums");
    category2->_width   = _parentWidget->width()-20;
    category2->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(category2);

    Column = 0;
    categorieRow++;
    int oldInfoSize = m_infosize;
    foreach (MEDIA::AlbumPtr album, _collectionModel->albumItemList)
    {
        if(!album->isFavorite)  continue;
        if(!_collectionModel->isAlbumFiltered(album)) continue;

        m_infosize++; // we count album favorite item

        AlbumGraphicItem *album_item = new AlbumGraphicItem();
        album_item->media = album;
        album_item->setPos(4+160*Column, categorieRow*50 + artistRow*165 + albumRow*170);
        addItem(album_item);

        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          albumRow++;
        }
    } // end album loop

    if(oldInfoSize == m_infosize) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text   = tr("No entry found");
      info->_width  = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(info);
    }
}



void CollectionScene::populatePlaylistOverviewScene()
{
    int categorieRow = 0;
    int playlistRow  = 0;
    int Column       = 0;
    m_infosize       = 0;
    int idx    = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();
    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Playlists") );
    header->setPos(0, 5);
    addItem(header);
    
    //! menu button    
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("new_playlist"));
    menu->addAction(m_actions.value("playlist_tracks"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    categorieRow++;

    /*-----------------------------------------------*/
    /* Playlist T_DATABASE                           */
    /* ----------------------------------------------*/
    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = tr("Playlist files from internal database");
    category->_width   = _parentWidget->width()-20;
    category->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(category);
    categorieRow++;

    //! playlist loop for T_DATABASE playlist
    int i = 0;
    for ( ; i < _plsFileModel->rootItem()->childCount(); i++ )
    {
      MEDIA::PlaylistPtr playlist = MEDIA::PlaylistPtr::staticCast(_plsFileModel->rootItem()->child(i));

      //WARNING drop smart playlist for this view
      if(playlist->p_type != T_DATABASE) break;

      if(!_plsFileModel->isPlaylistFiltered(playlist) ) continue;

      m_infosize++;
      idx++; // nouvelle ligne

      PlaylistGraphicItem *playlist_item = new PlaylistGraphicItem();
      playlist_item->media = playlist;
      playlist_item->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(playlist_item);

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }
    }

    if(m_infosize == 0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(info);
      categorieRow++;
    }

    int oldInfoSize = m_infosize;

    if(idx>0) playlistRow++;
    Column = 0;

    /*-----------------------------------------------*/
    /* Playlist T_FILE                               */
    /* ----------------------------------------------*/
    CategorieGraphicItem *category2 = new CategorieGraphicItem();
    category2->_catName = tr("Playlist files from user collection");
    category2->_width   =  _parentWidget->width() -20;
    category2->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(category2);
    categorieRow++;

    //! playlist loop for T_DATABASE playlist
    for ( ; i < _plsFileModel->rootItem()->childCount(); i++ )
    {
      MEDIA::PlaylistPtr playlist = MEDIA::PlaylistPtr::staticCast(_plsFileModel->rootItem()->child(i));

      //WARNING drop smart playlist for this view
      if(playlist->p_type != T_FILE) break;

      if(!_plsFileModel->isPlaylistFiltered(playlist) ) continue;

      m_infosize++;
      idx++; // nouvelle ligne

      PlaylistGraphicItem *playlist_item = new PlaylistGraphicItem();
      playlist_item->media = playlist;
      playlist_item->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(playlist_item);

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }
    }

    if(m_infosize == oldInfoSize) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(info);
    }
}


void CollectionScene::populatePlaylistByTrackScene()
{
    int categorieRow  = 0;
    int playlistRow   = 0;
    int trackRow      = 0;
    int trackPerAlbum = 0;
    int offset        = 0;
    m_infosize         = 0;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Playlists") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button    
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("new_playlist"));
    menu->addAction(m_actions.value("playlist_tracks"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);

    categorieRow++;


    //! playlist loop
    for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ )
    {
      MEDIA::PlaylistPtr playlist = MEDIA::PlaylistPtr::staticCast(_plsFileModel->rootItem()->child(i));

      //WARNING drop smart playlist for this view
      if(playlist->p_type == T_SMART) break;

      if(!_plsFileModel->isPlaylistFiltered(playlist) ) continue;

      CategorieGraphicItem *category = new CategorieGraphicItem();
      category->_catName = playlist->name;
      category->_width   = _parentWidget->width() -20;
      category->setPos( 0 , 10 + categorieRow*50 + playlistRow*30 + trackRow*20 + offset );
      addItem(category);

      categorieRow++;

      PlaylistGraphicItem *playlist_item = new PlaylistGraphicItem();
      playlist_item->setFlag(QGraphicsItem::ItemIsSelectable, false);
      playlist_item->media = playlist;
      playlist_item->setPos(10, categorieRow*50 + playlistRow*30 + trackRow*20 + offset );
      addItem(playlist_item);

      //! track loop
      trackPerAlbum = 0;
      for (int k = 0; k < playlist->childCount(); k++)
      {
        MEDIA::TrackPtr track = MEDIA::TrackPtr::staticCast(playlist->child(k));
        if(!_plsFileModel->isTrackFiltered(track)) continue;

        TrackGraphicItem *track_item = new TrackGraphicItem();
        track_item->media = track;
        track_item->setPos(155, categorieRow*50 + playlistRow*30 + trackRow*20 + offset);
        track_item->_width = _parentWidget->width()-155 -20;

        addItem(track_item);

        trackRow++;
        trackPerAlbum++;
        m_infosize++;
      }
      playlistRow++;
      if( trackPerAlbum < 6 ) offset = offset + (6 - trackPerAlbum)*25;
    }


    if(m_infosize==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , 10 + categorieRow*50 + playlistRow*150);
      addItem(info);
    }
}



void CollectionScene::populatePlaylistSmartScene()
{
    int categorieRow = 0;
    int playlistRow  = 0;
    int Column       = 0;
    m_infosize        = 0;
    int idx = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();
    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Smarts playlists") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button    
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("new_smart"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);
    
    categorieRow++;

    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = tr("Smart Playlists");
    category->_width   = _parentWidget->width() -20;
    category->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
    addItem(category);
    categorieRow++;


    //! playlist loop
    for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ )
    {
      MEDIA::PlaylistPtr playlist = MEDIA::PlaylistPtr::staticCast(_plsFileModel->rootItem()->child(i));
      if(playlist->p_type != T_SMART)
        continue;

      m_infosize++;
      idx++; // nouvelle ligne

      PlaylistGraphicItem *playlist_item = new PlaylistGraphicItem();
      playlist_item->media = playlist;
      playlist_item->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
      addItem(playlist_item);

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        playlistRow++;
        idx = 0;
      }
    }
}



void CollectionScene::populateHistoScene()
{
    Debug::debug() << "CollectionScene::populateHistoScene";

    int categorieRow  = 0;
    int trackRow      = 0;
    m_infosize        = 0;

    _histoModel->updateModel();

    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();
    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("History") );
    header->setPos(0, 5);
    addItem(header);    

    //! menu button    
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("clear_histo"));
    menu->addAction(m_actions.value("reload_histo"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);
    
    categorieRow++;

    //! track loop from history model
    QDate oldDate, newDate;
    for ( int i = 0; i < _histoModel->itemCount(); i++ )
    {
      if(!_histoModel->isItemFiltered(i)) continue;

      MEDIA::TrackPtr track = _histoModel->trackAt(i);

      newDate = QDateTime::fromTime_t(track->lastPlayed).date();

      if(oldDate != newDate)
      {
        oldDate = newDate;

        CategorieGraphicItem *category = new CategorieGraphicItem();
        category->_catName = track->lastplayed_ago();
        category->_width   = _parentWidget->width()-20;
        category->setPos(0 , 10 + categorieRow*50 + trackRow*20);
        addItem(category);

        categorieRow++;
      }

      TrackGraphicItem_v3 *track_item = new TrackGraphicItem_v3();
      track_item->media = track;
      track_item->setPos(30, categorieRow*50 + trackRow*20);
      track_item->_width = _parentWidget->width()-50 -20;

      if(track->type() == TYPE_STREAM)
        track_item->setToolTip(track->url);

      addItem(track_item);
      m_infosize++;

      trackRow++;
    }

    if (m_infosize == 0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 10 ,10 + categorieRow*50);
      addItem(info);
    }
}

/*******************************************************************************
 Mouse and menu event
*******************************************************************************/
void CollectionScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    Debug::debug() << "CollectionScene::contextMenuEvent";
    _mouseGrabbedItem = this->itemAt(event->scenePos());

    if(!_mouseGrabbedItem) {
      m_graphic_item_menu->hide();
      event->accept();
      return;
    }

    if(this->selectedItems().isEmpty() || !selectedItems().contains(_mouseGrabbedItem))
    {
      m_graphic_item_menu->setSelectedItems(QList<QGraphicsItem*>());
      m_graphic_item_menu->setGraphicItem(_mouseGrabbedItem);
    }
    else
    {
      m_graphic_item_menu->setSelectedItems(selectedItems());
      m_graphic_item_menu->setGraphicItem(0);
    }

    m_graphic_item_menu->updateMenu();
    m_graphic_item_menu->exec(event->screenPos());

    // no need to propagate to item
    event->accept();
}

void CollectionScene::slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU id)
{
    switch(id)
    {
      case ALBUM_PLAY          : playAlbum();              break;
      case ALBUM_LOAD_COVER    : loadCoverArtFromFile();   break;
      case ALBUM_LASTFM_COVER  : loadLastFmCover();        break;
      case ALBUM_REMOVE_COVER  : removeCoverArt();         break;
      case ALBUM_QUEUE_END     : enqueueAlbum(false);      break;
      case ALBUM_QUEUE_NEW     : enqueueAlbum(true);       break;
      case ALBUM_FAVORITE      : updateAlbumFavorite();    break;
      case GENRE_PLAY          : playAlbumGenre();         break;
      case GENRE_QUEUE_END     : enqueueAlbumGenre(false); break;
      case GENRE_QUEUE_NEW     : enqueueAlbumGenre(true);  break;
      case ARTIST_PLAY         : playArtist();             break;
      case ARTIST_QUEUE_END    : enqueueArtist(false);     break;
      case ARTIST_QUEUE_NEW    : enqueueArtist(true);      break;
      case ARTIST_FAVORITE     : updateArtistFavorite();   break;
      case PLAYLIST_PLAY       : playPlaylist();           break;
      case PLAYLIST_EDIT       : edit_playlist_dialog();   break;
      case PLAYLIST_QUEUE_END  : enqueuePlaylist(false);   break;
      case PLAYLIST_QUEUE_NEW  : enqueuePlaylist(true);    break;
      case PLAYLIST_FAVORITE   : updatePlaylistFavorite(); break;
      case PLAYLIST_REMOVE     : removePlaylistFromDisk(); break;
      case TRACK_PLAY          : playTrack();              break;
      case TRACK_QUEUE_END     : enqueueTrack(false);      break;
      case TRACK_QUEUE_NEW     : enqueueTrack(true);       break;

      case ALBUM_EDIT          :
      case GENRE_EDIT          :
      case ARTIST_EDIT         :
      case TRACK_EDIT          : edit_media_dialog();      break;

      case SELECTION_PLAY      : playSelected();break;
      case SELECTION_QUEUE_END : enqueueSelected(false);break;
      case SELECTION_QUEUE_NEW : enqueueSelected(true);break;

      default:break;
    }
}


void CollectionScene::slot_item_mouseMove()
{
    //Debug::debug() << "CollectionScene::slot_item_mouseMove";

    // single drag and drop
    QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (ACTIONS()->value(BROWSER_ITEM_MOUSE_MOVE))->data() );

    if(this->selectedItems().isEmpty() || !selectedItems().contains(gItem))
    {
      switch(gItem->type()) {
        case GraphicsItem::AlbumType      : startAlbumsDrag(gItem);break;
        case GraphicsItem::AlbumGenreType : startAlbumsGenreDrag(gItem);break;
        case GraphicsItem::ArtistType     : startArtistsDrag(gItem);break;
        case GraphicsItem::TrackType      : startTracksDrag(gItem);break;
        case GraphicsItem::PlaylistType   : startPlaylistsDrag(gItem);break;
      }
    }
    // multiple drag and drop
    else
    {
      switch(gItem->type()) {
        case GraphicsItem::AlbumType      : startAlbumsDrag();break;
        case GraphicsItem::AlbumGenreType : startAlbumsGenreDrag();break;
        case GraphicsItem::ArtistType     : startArtistsDrag();break;
        case GraphicsItem::TrackType      : startTracksDrag();break;
        case GraphicsItem::PlaylistType   : startPlaylistsDrag();break;
      }
    }
}


void CollectionScene::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event )
{
    //Debug::debug() << "CollectionScene::mouseDoubleClickEvent";
    // A FAIRE EGALEMENT DANS LES AUTRES SCENE SI CELA FONCTIONNE
    _mouseGrabbedItem = this->itemAt(event->scenePos());

    if(!_mouseGrabbedItem) {
      event->ignore();
      return;
    }

    // ENQUEUE
    if(Qt::ControlModifier == QApplication::keyboardModifiers())
    {
      switch(_mouseGrabbedItem->type()) {
        case GraphicsItem::AlbumType      : enqueueAlbum(false);break;
        case GraphicsItem::AlbumGenreType : enqueueAlbumGenre(false);break;
        case GraphicsItem::ArtistType     : enqueueArtist(false);break;
        case GraphicsItem::TrackType      : enqueueTrack(false);break;
        case GraphicsItem::PlaylistType   : enqueuePlaylist(false);break;
        default: event->ignore(); break;
      }
    }
    // PLAY ITEM
    else
    {
        switch(_mouseGrabbedItem->type()) {
          case GraphicsItem::AlbumType      : playAlbum();break;
          case GraphicsItem::AlbumGenreType : playAlbumGenre();break;
          case GraphicsItem::ArtistType     : playArtist();break;
          case GraphicsItem::TrackType      : playTrack();break;
          case GraphicsItem::PlaylistType   : playPlaylist();break;
          default: event->ignore(); break;
        }
    }

    // no need to propagate to item
    event->accept();
}



//! --------------------- slot_item_ratingclick --------------------------------
void CollectionScene::slot_item_ratingclick()
{
    QList<QGraphicsView *> view = this->views();
    if( view.isEmpty() )
      return;

    //Debug::debug() << "CollectionScene::slot_item_ratingclick";
    QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (ACTIONS()->value(BROWSER_ITEM_RATING_CLICK))->data() );

    switch(gItem->type()) {
      case GraphicsItem::AlbumType      :
      case GraphicsItem::AlbumGenreType : rateAlbum(gItem);break;
      case GraphicsItem::ArtistType     : rateArtist(gItem);break;
      case GraphicsItem::TrackType      : rateTrack(gItem);break;
      default: return;break;
    }
}

/*******************************************************************************
  CollectionScene::sortedSelectedItem
*******************************************************************************/
QList<QGraphicsItem*> CollectionScene::sortedSelectedItem()
{
    QList<QGraphicsItem*> selected = selectedItems();
    int count = selected.count();

    bool sort_ok = false;

    do
    {
      sort_ok = false;
      for(int i =0; i<=count-2; i++)
      {
        QGraphicsItem* gi1 = selected.at(i);
        QGraphicsItem* gi2 = selected.at(i+1);

        QPoint p1 = gi1->scenePos().toPoint();
        QPoint p2 = gi2->scenePos().toPoint();

        bool lessthan;
        if(p1.y() < p2.y())
          lessthan = true;
        else if (p1.y() > p2.y())
          lessthan = false;
        else if (p1.x() < p2.x())
          lessthan = true;
        else
          lessthan = false;

        if(!lessthan) {
          selected.swap(i,i+1);
          sort_ok = true;
        }
      } // end for
    } while(sort_ok);

    return selected;
}


/*******************************************************************************
  CollectionScene::get_item_position
*******************************************************************************/
QPoint CollectionScene::get_item_position(MEDIA::MediaPtr media)
{
    Debug::debug() << "CollectionScene::get_item_position";
    QPoint point = QPoint(0,0);
    if(media->type() == TYPE_ARTIST && m_view_mode == MAIN::ViewArtist)
    {
      foreach(QGraphicsItem* gItem, this->items()) {
        if(gItem->type() != GraphicsItem::ArtistType) continue;

        ArtistGraphicItem* artist_item = static_cast<ArtistGraphicItem*>(gItem);

        if(artist_item->media->id == MEDIA::ArtistPtr::staticCast(media)->id) {
          point = artist_item->scenePos().toPoint();
          break;
        }
      }
    }
    else if(media->type() == TYPE_ALBUM && m_view_mode == MAIN::ViewAlbum)
    {
      foreach(QGraphicsItem* gItem, this->items()) {
        if(gItem->type() != GraphicsItem::AlbumType) continue;

        AlbumGraphicItem* album_item = static_cast<AlbumGraphicItem*>(gItem);

        if(album_item->media->id == MEDIA::AlbumPtr::staticCast(media)->id) {
          point = album_item->scenePos().toPoint();
          break;
        }
      }
    }
    else if(media->type() == TYPE_TRACK && m_view_mode == MAIN::ViewTrack)
    {
      foreach(QGraphicsItem* gItem, this->items()) {
        if(gItem->type() != GraphicsItem::TrackType) continue;

        TrackGraphicItem* track_item = static_cast<TrackGraphicItem*>(gItem);

        if(track_item->media->id == MEDIA::TrackPtr::staticCast(media)->id) {
          point = track_item->scenePos().toPoint();
          break;
        }
      }
    }

    return point;
}


/*******************************************************************************
    album cover stuff
*******************************************************************************/
void CollectionScene::loadCoverArtFromFile()
{
    //Debug::debug() << "CollectionScene::loadCoverArtFromFile";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    //! open file dialog
    QString filename = QFileDialog::getOpenFileName(new QWidget(), tr("Select cover file..."),tr("Files (*.png *.jpg)"));
    if( filename.isEmpty() ) return;

    const QString cover_path = UTIL::CONFIGDIR + "/albums/" + item->media->coverpath;

    //! get new cover file
    QImage image = QImage(filename);
    image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);

    //! remove existing cover file
    if(QFile::exists(cover_path))
    {
      QFile::remove(cover_path);
      CoverCache::instance()->invalidate(item->media);
    }
    //! save new cover file
    image.save(cover_path, "png", -1);

    //! scene update
    item->update();
}

void CollectionScene::loadLastFmCover()
{
    //Debug::debug() << "CollectionScene::loadLastFmCover";

    if(!_mouseGrabbedItem) return;

    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    //! Get album Name and coverPath
    const QString album      =  item->media->name;
    MEDIA::ArtistPtr artist_ptr = MEDIA::ArtistPtr::staticCast(item->media->parent());
    const QString artist     =  artist_ptr->name;

    //! start search cover process
    ThreadManager::instance()->startCoverSearch(artist, album);
}

void CollectionScene::removeCoverArt()
{
    //Debug::debug() << "CollectionScene::RemoveCoverArtForAlbum";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    const QString cover_path =  UTIL::CONFIGDIR + "/albums/" + item->media->coverpath;

    //! remove existing cover file
    if(QFile::exists(cover_path))
    {
      QFile::remove(cover_path);
      CoverCache::instance()->invalidate(item->media);
      item->update();
    }
}


/*******************************************************************************
    Edit media dialog
*******************************************************************************/
void CollectionScene::edit_media_dialog()
{
    //Debug::debug() << "CollectionScene::edit_media_dialog ";
    MediaItem_Edit_Dialog *dialog = 0;

    if(!_mouseGrabbedItem) return;

    if(_mouseGrabbedItem->type()== GraphicsItem::ArtistType)
    {
      ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_ARTIST);
      dialog->setMediaItem(item->media);
    }
    else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumType)
    {
      AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_ALBUM);
      dialog->setMediaItem(item->media);
    }
    else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumGenreType)
    {
      AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_GENRE);
      dialog->setGenre(item->_genre);
      dialog->setMediaItem(item->media);
    }
    else if(_mouseGrabbedItem->type()== GraphicsItem::TrackType)
    {
      TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);
      dialog = new MediaItem_Edit_Dialog(DIALOG_TRACK);
      dialog->setMediaItem(item->media);
    }

    if(dialog) {
      //! blocking call (modal dialog)
      dialog->exec();

      if( (dialog->result() == QDialog::Accepted) && dialog->isChanged()) {
        ThreadManager::instance()->populateCollectionModel();
      }
      dialog->deleteLater();
    }
}


/*******************************************************************************
    Update favorite status
*******************************************************************************/
void CollectionScene::updateAlbumFavorite()
{
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

    int _albumBdId    = item->media->id;
    bool isFavorite   = item->media->isFavorite;

    item->media->isFavorite = !isFavorite;

    Database db;
    if (!db.connect()) return;

    QSqlQuery query("UPDATE `albums` "       \
                    "SET `favorite`="+QString::number(!isFavorite)+" " \
                    "WHERE `id`="+QString::number(_albumBdId)+";",*db.sqlDb());
    query.exec();

    //! item update
    item->update();
}


void CollectionScene::updateArtistFavorite()
{
    if(!_mouseGrabbedItem) return;
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

    int _artistBdId = item->media->id;
    bool isFavorite = item->media->isFavorite;

    item->media->isFavorite = !isFavorite;

    Database db;
    if (!db.connect()) return;

    QSqlQuery query("UPDATE `artists` "       \
                     "SET `favorite`="+QString::number(!isFavorite)+" " \
                     "WHERE `id`="+QString::number(_artistBdId)+";",*db.sqlDb());
    query.exec();

    //! item update
    item->update();
}

/*******************************************************************************
    Playlist stuff
*******************************************************************************/
void CollectionScene::removePlaylistFromDisk()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data
    const int dbId       = item->media->id;
    const QString fname  = item->media->url;
    const int type       = int(item->media->p_type);

    //Debug::debug() << "CollectionScene:: remove playlist item id" << dbId;

    //! playlist is a local file on disk --> remove file
    if(type == T_FILE) {
      QFile playlistFile(fname);
      if(playlistFile.exists())
        playlistFile.remove();
    }

    //! remove from database
    Database db;
    if (!db.connect()) return;

    QSqlField data("col",QVariant::String);
    data.setValue(dbId);
    QString id = db.sqlDb()->driver()->formatValue(data,false);

    if(type == T_DATABASE)
    {
      QSqlQuery query("DELETE FROM `playlists` WHERE `id`="+id+";",*db.sqlDb());
      query.exec();

      QSqlQuery query2("DELETE FROM `playlist_items` WHERE `playlist_id` NOT IN (SELECT `id` FROM `playlists`);", *db.sqlDb());
      query2.exec();
    }
    else //T_SMART
    {
      QSqlQuery query("DELETE FROM `smart_playlists` WHERE `id`="+id+";",*db.sqlDb());
      query.exec();
    }

    ThreadManager::instance()->populatePlsFileModel();
}


void CollectionScene::edit_playlist_dialog()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    const int type       = int(item->media->p_type);

    if(type == T_DATABASE || type == T_FILE)
    {
      PlaylistEditor *editor = new PlaylistEditor(_parentWidget);
      editor->setPlaylist(item->media);
      editor->show();

      connect(editor, SIGNAL(closed()), this, SLOT(slot_delete_playlist_editor()));
    }
    else //T_SMART
    {
      // open smart playlist editor with current smart playlist
      if( SmartPlaylist::edit_dialog( item->media ) )
        ThreadManager::instance()->populatePlsFileModel();
    }
}

/*******************************************************************************
    Playlist creation
*******************************************************************************/
void CollectionScene::slot_create_new_smart_playlist()
{
   // open smart playlist editor for a new one
   if( SmartPlaylist::create_dialog() )
     ThreadManager::instance()->populatePlsFileModel();
}

void CollectionScene::slot_create_new_playlist()
{
    PlaylistEditor *editor = new PlaylistEditor(_parentWidget);
    editor->show();

    connect(editor, SIGNAL(closed()), this, SLOT(slot_delete_playlist_editor()));
}


void CollectionScene::slot_delete_playlist_editor()
{
    PlaylistEditor *editor = qobject_cast<PlaylistEditor *>(sender());
    if(editor)
      editor->deleteLater();
}


void CollectionScene::updatePlaylistFavorite()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data
    const int dbId       = item->media->id;
    const int type       = int(item->media->p_type);
    bool isFavorite      = item->media->isFavorite;

    item->media->isFavorite = !isFavorite;

    Database db;
    if (!db.connect()) return;

    if(type == T_DATABASE || type == T_FILE)
    {
      QSqlQuery query("UPDATE `playlists` "       \
                      "SET `favorite`="+QString::number(!isFavorite)+" " \
                      "WHERE `id`="+QString::number(dbId)+";",*db.sqlDb());
      query.exec();
    }
    else //T_SMART
    {
      QSqlQuery query("UPDATE `smart_playlists` "       \
                      "SET `favorite`="+QString::number(!isFavorite)+" " \
                      "WHERE `id`="+QString::number(dbId)+";",*db.sqlDb());
      query.exec();
    }

    //! item update
    item->update();
}

/*******************************************************************************
    History model clear
*******************************************************************************/
void CollectionScene::slot_clear_history()
{
    HistoManager::instance()->clearHistory();
    populateScene();
}

